@@ -1,21 +1,12 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+#include <stdio.h>
+#include <time.h>
+
 #include "oafd.h"
 #include "liboaf/liboaf.h"
 #include "ac-query-expr.h"
-#include <time.h>
 
-static void
-OAF_ServerInfo__copy (OAF_ServerInfo * new, const OAF_ServerInfo * old)
-{
-	new->iid = CORBA_string_dup (old->iid);
-	new->server_type = CORBA_string_dup (old->server_type);
-	new->location_info = CORBA_string_dup (old->location_info);
-	new->username = CORBA_string_dup ("");
-	new->hostname = CORBA_string_dup ("");
-	new->domain = CORBA_string_dup ("");
-	new->attrs = old->attrs;
-	CORBA_sequence_set_release (&new->attrs, CORBA_FALSE);
-}
+#define OAF_LINK_TIME_TO_LIVE 256
 
 typedef struct
 {
@@ -251,13 +242,13 @@ static PortableServer_ServantBase__epv impl_OAF_ActivationContext_base_epv = {
 };
 static POA_OAF_ActivationContext__epv impl_OAF_ActivationContext_epv = {
 	NULL,			/* _private */
-	&impl_OAF_ActivationContext__get_directories,
-	&impl_OAF_ActivationContext_add_directory,
-	&impl_OAF_ActivationContext_remove_directory,
-	&impl_OAF_ActivationContext_activate,
-	&impl_OAF_ActivationContext__get_servers,
-	&impl_OAF_ActivationContext_query,
-	&impl_OAF_ActivationContext_activate_from_id
+	(gpointer) &impl_OAF_ActivationContext__get_directories,
+	(gpointer) &impl_OAF_ActivationContext_add_directory,
+	(gpointer) &impl_OAF_ActivationContext_remove_directory,
+	(gpointer) &impl_OAF_ActivationContext_activate,
+	(gpointer) &impl_OAF_ActivationContext__get_servers,
+	(gpointer) &impl_OAF_ActivationContext_query,
+	(gpointer) &impl_OAF_ActivationContext_activate_from_id
 };
 
 /*** vepv structures ***/
@@ -418,50 +409,63 @@ ac_do_activation (impl_POA_OAF_ActivationContext * servant,
 	}
 
 	for (num_layers = 0, activatable = server;
-	     activatable && !strcmp (activatable->server_type, "factory");
-	     num_layers++) {
-		activatable =
-			g_hash_table_lookup (child->by_iid,
-					     activatable->location_info);
+             activatable && !strcmp (activatable->server_type, "factory") &&
+             num_layers < OAF_LINK_TIME_TO_LIVE; num_layers++) {
+
+		activatable = g_hash_table_lookup (child->by_iid, activatable->location_info);
 	}
 
-	if (activatable == NULL) {
+	if (activatable == NULL) {		
 		OAF_GeneralError *errval = OAF_GeneralError__alloc ();
-		errval->description =
-			CORBA_string_dup ("Couldn't find the factory server");
+		errval->description = CORBA_string_dup ("Couldn't find the factory server");
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+				     ex_OAF_GeneralError, errval);
+		return;
+	} 
+	else if (num_layers == OAF_LINK_TIME_TO_LIVE) {
+		OAF_GeneralError *errval = OAF_GeneralError__alloc ();
+		errval->description = CORBA_string_dup ("Location loop");
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_OAF_GeneralError, errval);
 		return;
-	}
+        }
 
 	/* A shared library must be on the same host as the activator in
 	 * order for loading to work properly (no, we're not going to
 	 * bother with loading a remote shlib into a process - it gets far too complicated
 	 * far too quickly :-) */
+	
 	if (activatable && !strcmp (activatable->server_type, "shlib")
 	    && !(flags & OAF_FLAG_NO_LOCAL)
 	    && (hostname && !strcmp (activatable->hostname, hostname))) {
 		int j;
 		char tbuf[512];
+		
+		out->res._d = OAF_RESULT_SHLIB;		
 
-		out->res._d = OAF_RESULT_SHLIB;
-		out->res._u.res_shlib._length = num_layers + 1;
-		out->res._u.res_shlib._buffer =
-			CORBA_sequence_CORBA_string_allocbuf (num_layers);
+		/* Here is an explanation as to why we add 2 to num_layers.
+		 * At the end of the string list, after all the factory iids are added
+		 * to the string list, we then add the iid of the shaed library and the 
+		 * location info.  This data is later used in oaf_server_activate_shlib
+		 * to activate the component
+		 */		 
+		out->res._u.res_shlib._length = num_layers + 2;
+		out->res._u.res_shlib._buffer = CORBA_sequence_CORBA_string_allocbuf (num_layers + 2);
 
-		for (j = 0, activatable = server;
-		     activatable
+		/* Copy over factory info */
+		for (j = 0, activatable = server; activatable
 		     && !strcmp (activatable->server_type, "factory"); j++) {
-			out->res._u.res_shlib._buffer[j] =
-				CORBA_string_dup (activatable->iid);
-			activatable =
-				g_hash_table_lookup (child->by_iid,
-						     activatable->location_info);
+			out->res._u.res_shlib._buffer[j] = CORBA_string_dup (activatable->iid);
+			activatable = g_hash_table_lookup (child->by_iid,
+						     	   activatable->location_info);
 		}
 
-		out->res._u.res_shlib._buffer[j] =
-			CORBA_string_dup (activatable->iid);
+		/* Copy shlib iid into buffer */
+		out->res._u.res_shlib._buffer[j] = CORBA_string_dup (activatable->iid);
 
+		/* Copy location into last buffer slot for use in later activation */
+		out->res._u.res_shlib._buffer[j+1] = CORBA_string_dup (activatable->location_info);
+		
 		g_snprintf (tbuf, sizeof (tbuf), "OAFAID:[%s,%s,%s,%s]",
 			    activatable->iid,
 			    activatable->username,
@@ -503,16 +507,20 @@ ctx_get_value (CORBA_Context ctx, const char *propname,
 		if (nvout->list->len > 0) {
 			CORBA_NamedValue *nv;
 
-			nv =
-				&g_array_index (nvout->list, CORBA_NamedValue,
-						0);
+			nv = &g_array_index (nvout->list, CORBA_NamedValue, 0);
 			retval = g_strdup (*(char **) nv->argument._value);
 		} else
 			retval = NULL;
 
 		CORBA_NVList_free (nvout, ev);
 	} else
+        {                
+                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+                                     ex_OAF_ActivationContext_IncompleteContext,
+                                     NULL);
+                
 		retval = NULL;
+        }
 
 	return retval;
 }
@@ -535,14 +543,13 @@ impl_OAF_ActivationContext_activate (impl_POA_OAF_ActivationContext * servant,
 
 	servant->refs++;
 
-	items =
-		oaf_alloca (servant->total_servers *
+	items = oaf_alloca (servant->total_servers *
 			    sizeof (OAF_ServerInfo *));
 	ac_query_run (servant, requirements, selection_order, ctx, items, ev);
 
 	if (ev->_major != CORBA_NO_EXCEPTION) {
 		goto out;
-	}
+        }
 
 	retval = OAF_ActivationResult__alloc ();
 	retval->res._d = OAF_RESULT_NONE;
@@ -605,9 +612,9 @@ impl_OAF_ActivationContext__get_servers (impl_POA_OAF_ActivationContext *
 			child = cur->data;
 
 			for (j = 0; j < child->list->_length; j++, i++)
-				OAF_ServerInfo__copy (&retval->_buffer[i],
-						      &child->
-						      list->_buffer[j]);
+				OAF_ServerInfo_copy (&retval->_buffer[i],
+						     &child->
+						     list->_buffer[j]);
 		}
 	}
 
@@ -802,7 +809,7 @@ impl_OAF_ActivationContext_query (impl_POA_OAF_ActivationContext * servant,
 			if (!items[i])
 				continue;
 
-			OAF_ServerInfo__copy (&retval->_buffer[j], items[i]);
+			OAF_ServerInfo_copy (&retval->_buffer[j], items[i]);
 
 			j++;
 		}
@@ -821,11 +828,18 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
 					     CORBA_Environment * ev)
 {
 	OAF_ActivationResult *retval;
-	GSList *cur;
 	OAFActivationInfo *ainfo;
-	ChildODInfo *child = NULL;
-	OAF_ServerInfo *si;
-	char *hostname;
+	char *context_username;
+	char *context_hostname;
+	char *context_domain;
+        char *tmp_aid;
+        char *requirements;
+        char *iid_requirement;
+        char *username_requirement;
+        char *hostname_requirement;
+        char *domain_requirement;
+        char *sort_criteria[4];
+        GNOME_stringlist selection_order;
 
 	ac_update_lists (servant, ev);
 
@@ -834,37 +848,86 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
 	retval = OAF_ActivationResult__alloc ();
 	retval->res._d = OAF_RESULT_NONE;
 
-	ainfo = oaf_actid_parse (aid);
+        if (strncmp ("OAFIID:", aid, 7) == 0) {
+                tmp_aid = g_strconcat ("OAFAID:[", aid, "]", NULL);
+                ainfo = oaf_actid_parse (tmp_aid);
+                g_free (tmp_aid);
+        } else {
+                ainfo = oaf_actid_parse (aid);
+        }
 
 	if (!ainfo) {
-		if (!strncmp ("OAFIID:", aid, 7)) {
-			static const char *query_prefix = "iid == \'";
-			static const char *query_suffix = "\'";
-			char *requirements;
-			GNOME_stringlist selection_order;
-
-			requirements = g_malloc (strlen (query_prefix) +
-						 strlen (aid) +
-						 strlen (query_suffix) + 1);
-			strcpy (requirements, query_prefix);
-			strcat (requirements, aid);
-			strcat (requirements, query_suffix);
-
-			selection_order._length = 0;
-			selection_order._buffer = NULL;
-
-			retval =
-				impl_OAF_ActivationContext_activate (servant,
-								     requirements,
-								     &selection_order,
-								     flags,
-								     ctx, ev);
-			goto out;
-		} else {
-			goto out;
-		}
+                goto out;
 	}
 
+        iid_requirement = g_strconcat ("iid == \'", ainfo->iid, "\' ", NULL);
+
+        if (ainfo->user) {
+                username_requirement = g_strconcat ("AND username == \'", ainfo->user, "\'", NULL);
+        } else {
+                username_requirement = g_strdup ("");
+        }
+        
+        if (ainfo->host) {
+                hostname_requirement = g_strconcat ("AND hostname == \'", ainfo->host, "\'", NULL);
+        } else {
+                hostname_requirement = g_strdup ("");
+        }
+        
+        if (ainfo->domain) {
+                domain_requirement = g_strconcat ("AND domain == \'", ainfo->domain, "\'", NULL);
+        } else {
+                domain_requirement = g_strdup ("");
+        }
+
+        requirements = g_strconcat (iid_requirement, username_requirement, 
+                                    hostname_requirement, domain_requirement, NULL);
+
+        g_free (iid_requirement);
+        g_free (username_requirement);
+        g_free (hostname_requirement);
+        g_free (domain_requirement);
+
+        /* FIXME: either I am doing something really wrong here or
+           CORBA_Context is broken in ORBit */
+
+        context_username = ctx_get_value (ctx, "username", ev);
+        context_hostname = ctx_get_value (ctx, "hostname", ev);
+        context_domain = ctx_get_value (ctx, "domain", ev);
+
+	if (ev->_major != CORBA_NO_EXCEPTION)
+        {
+                goto out;
+        }
+        
+        sort_criteria[0] = g_strconcat ("username == \'", context_username, "\'", NULL);
+        sort_criteria[1] = g_strconcat ("hostname == \'", context_hostname, "\'", NULL);
+        sort_criteria[2] = g_strconcat ("session == \'", context_domain, "\'", NULL);
+        sort_criteria[3] = NULL;
+
+        g_free (context_username);
+        g_free (context_hostname);
+        g_free (context_domain);
+
+        selection_order._length = 3;
+        selection_order._buffer = sort_criteria;
+        CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
+        
+        retval =
+                impl_OAF_ActivationContext_activate (servant,
+                                                     requirements,
+                                                     &selection_order,
+                                                     flags,
+                                                     ctx, ev);
+
+        g_free (sort_criteria[0]);
+        g_free (sort_criteria[1]);
+        g_free (sort_criteria[2]);
+
+        goto out;
+
+#if 0
+
 	for (cur = servant->dirs; cur && !child; cur = cur->next) {
 		ChildODInfo *curchild;
 
@@ -886,19 +949,31 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
 	}
 
 	if (!child)
+        {
+                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+                                     ex_OAF_ActivationContext_IncompleteContext,
+                                     NULL);
 		goto out;	/* XXX in future, add hook to allow starting a new OD on demand */
+        }
+
+
 
-	si = g_hash_table_lookup (child->by_iid, ainfo->iid);
 	if (!si)
 		goto out;
 
 	hostname = ctx_get_value (ctx, "hostname", ev);
 	if (ev->_major != CORBA_NO_EXCEPTION)
+        {
+                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+                                     ex_OAF_ActivationContext_IncompleteContext,
+                                     NULL);
 		goto out;
+        }
 
 	ac_do_activation (servant, si, retval, flags, hostname, ctx, ev);
 
 	g_free (hostname);
+#endif
 
       out:
 	servant->refs--;
@@ -914,6 +989,5 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
 		retval = NULL;
 	}
 
-
 	return retval;
 }
