@@ -7,6 +7,7 @@
 
 #include "ac-query-expr.h"
 #include "ac-query-parse.h"
+#include "qsort_ex.h"
 
 static QueryExpr *
 qexp_new (void)
@@ -634,8 +635,6 @@ static QueryExprConst
 qexp_func_defined (OAF_ServerInfo * si, QueryExpr * e, QueryContext * qctx)
 {
 	QueryExprConst retval, v1;
-	char **check_one, *check_two;
-	int i;
 
 	v1 = qexp_evaluate (si, e->u.function_value.arguments->data, qctx);
 
@@ -767,11 +766,11 @@ qexp_evaluate_id (OAF_ServerInfo * si, QueryExpr * e, QueryContext * qctx)
 		else if (!strcasecmp (e->u.id_value, "iid"))
 			retval.u.v_string = si->iid;
 		else if (!strcasecmp (e->u.id_value, "username"))
-			retval.u.v_string = si->iid;
+			retval.u.v_string = si->username;
 		else if (!strcasecmp (e->u.id_value, "hostname"))
-			retval.u.v_string = si->iid;
+			retval.u.v_string = si->hostname;
 		else if (!strcasecmp (e->u.id_value, "domain"))
-			retval.u.v_string = si->iid;
+			retval.u.v_string = si->domain;
 		else {
 			int i;
 			for (i = 0; i < si->attrs._length; i++) {
@@ -1113,8 +1112,6 @@ void
 qexp_sort (OAF_ServerInfo ** servers, int nservers, QueryExpr ** sexps,
 	   int nexps, QueryContext * qctx)
 {
-	int n, h, i, j;
-	OAF_ServerInfo *t;
 	QexpSortData sort_data;
 
 	sort_data.sexps = sexps;
@@ -1122,5 +1119,5 @@ qexp_sort (OAF_ServerInfo ** servers, int nservers, QueryExpr ** sexps,
 	sort_data.qctx = qctx;
 
 	qsort_ex (servers, nservers, sizeof (OAF_ServerInfo *),
-		  qexp_sort_compare, &sort_data);
+		  (compar_ex_fn_t)qexp_sort_compare, &sort_data);
 }
