@@ -19,7 +19,10 @@ static void debug_queries (void);
 
 /* Option values */
 static char *od_source_dir = NULL;
-static char *ac_evaluate = NULL, *od_domain = "session";
+#ifdef OAF_DEBUG
+static char *ac_evaluate = NULL;
+#endif
+static char *od_domain = "session";
 static int server_ac = 0, ior_fd = -1, server_ns = 1;
 
 static struct poptOption options[] = {
@@ -67,7 +70,7 @@ main (int argc, char *argv[])
 
 	CORBA_exception_init (&ev);
 
-	ctx = poptGetContext ("oafd", argc, argv, options, 0);
+	ctx = poptGetContext ("oafd", argc, (const char **)argv, options, 0);
 	while (poptGetNextOpt (ctx) >= 0)
 		/**/;
 
@@ -83,7 +86,7 @@ main (int argc, char *argv[])
 		GString *real_od_source_dir;
 
 		real_od_source_dir = g_string_new (OAFINFODIR);
-		env_od_source_dir = getenv ("OAF_OD_SOURCE_DIR");
+		env_od_source_dir = getenv ("OAF_INFO_PATH");
 
 		if (od_source_dir) {
 			g_string_append_c (real_od_source_dir, ':');
@@ -95,20 +98,20 @@ main (int argc, char *argv[])
 					 env_od_source_dir);
 		}
 
-		od =
-			OAF_ObjectDirectory_create (root_poa, od_domain,
-						    real_od_source_dir->str,
-						    &ev);
+		od = OAF_ObjectDirectory_create (root_poa, od_domain,
+                                                 real_od_source_dir->str,
+                                                 &ev);
 		if (server_ns) {
 			CORBA_Object naming_service;
 
 			naming_service =
 				impl_CosNaming_NamingContext__create
 				(root_poa, &ev);
-			OAF_ObjectDirectory_register_new (od,
-							  "OAFIID:OAFNamingService:20000410",
-							  naming_service,
-							  &ev);
+			OAF_ObjectDirectory_register_new 
+                                (od,
+                                 "OAFIID:oaf_naming_service:7e2b90ef-eaf0-4239-bb7c-812606fcd80d",
+                                 naming_service,
+                                 &ev);
 		}
 
 		g_string_free (real_od_source_dir, TRUE);
