@@ -1,9 +1,14 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 #include "config.h"
 
+#include <stdio.h>
+
 #include "oafd.h"
 #include "liboaf/liboaf.h"
 #include <time.h>
+#include <glib.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 /*** App-specific servant structures ***/
 
@@ -24,6 +29,10 @@ typedef struct
 	GHashTable *active_servers;
 	OAF_CacheTime time_active_changed;
 
+        char **registry_source_directories;
+        time_t time_did_stat;
+        GHashTable *registry_directory_mtimes;
+
 	CORBA_Object self;
 }
 impl_POA_OAF_ObjectDirectory;
@@ -89,16 +98,16 @@ static PortableServer_ServantBase__epv impl_OAF_ObjectDirectory_base_epv = {
 };
 static POA_OAF_ObjectDirectory__epv impl_OAF_ObjectDirectory_epv = {
 	NULL,			/* _private */
-	&impl_OAF_ObjectDirectory__get_servers,
-	&impl_OAF_ObjectDirectory_get_active_servers,
-	&impl_OAF_ObjectDirectory__get_username,
-	&impl_OAF_ObjectDirectory__get_hostID,
-	&impl_OAF_ObjectDirectory__get_domain,
-	&impl_OAF_ObjectDirectory_activate,
-	&impl_OAF_ObjectDirectory_lock,
-	&impl_OAF_ObjectDirectory_unlock,
-	&impl_OAF_ObjectDirectory_register,
-	&impl_OAF_ObjectDirectory_unregister
+	(gpointer) &impl_OAF_ObjectDirectory__get_servers,
+	(gpointer) &impl_OAF_ObjectDirectory_get_active_servers,
+	(gpointer) &impl_OAF_ObjectDirectory__get_username,
+	(gpointer) &impl_OAF_ObjectDirectory__get_hostID,
+	(gpointer) &impl_OAF_ObjectDirectory__get_domain,
+	(gpointer) &impl_OAF_ObjectDirectory_activate,
+	(gpointer) &impl_OAF_ObjectDirectory_lock,
+	(gpointer) &impl_OAF_ObjectDirectory_unlock,
+	(gpointer) &impl_OAF_ObjectDirectory_register,
+	(gpointer) &impl_OAF_ObjectDirectory_unregister
 };
 
 /*** vepv structures ***/
@@ -160,10 +169,77 @@ od_dump_list (impl_POA_OAF_ObjectDirectory * od)
 }
 #endif
 
+static gboolean
+registry_directory_needs_update (impl_POA_OAF_ObjectDirectory *servant,
+                                 const char *directory)
+{
+        struct stat statbuf;
+        time_t old_mtime;
+
+        if (stat (directory, &statbuf) != 0) {
+                return FALSE;
+        }
+ 
+        old_mtime = (time_t) g_hash_table_lookup (servant->registry_directory_mtimes,
+                                                  directory);
+
+        g_hash_table_insert (servant->registry_directory_mtimes,
+                             (gpointer) directory,
+                             (gpointer) statbuf.st_mtime);
+
+        return (old_mtime != statbuf.st_mtime);
+}
+
+static void
+update_registry (impl_POA_OAF_ObjectDirectory *servant)
+{
+        gboolean must_load;
+        int i;
+
+        /* FIXME: we should only reload those directories that have
+         * actually changed instead of reloading all when any has
+         * changed. 
+         */
+
+        must_load = FALSE;
+
+        /* Don't stat more than once every 5 seconds or activation
+           could be too slow. This works even on the first read
+           because then `time_read is 0' */
+        if (time (NULL) - 5 > servant->time_did_stat) {
+                for (i = 0; servant->registry_source_directories[i] != NULL; i++) {
+                        if (registry_directory_needs_update 
+                            (servant, servant->registry_source_directories[i])) {
+                                must_load = TRUE;
+                                break;
+                        }
+                }
+
+                servant->time_did_stat = time (NULL);
+        }
+
+        if (must_load) {
+                puts ("reload");
+
+                servant->attr_servers._buffer =
+                        OAF_ServerInfo_load (servant->registry_source_directories,
+                                             &servant->attr_servers._length,
+                                             &servant->by_iid,
+                                             g_get_user_name (),
+                                             servant->attr_hostID,
+                                             servant->attr_domain);
+                servant->time_list_changed = time (NULL);
+
+#if defined(OAF_DEBUG)
+                od_dump_list (servant);
+#endif
+        }
+}
+
 OAF_ObjectDirectory
 OAF_ObjectDirectory_create (PortableServer_POA poa,
 			    const char *domain,
-			    const char *source_directory,
+			    const char *registry_path,
 			    CORBA_Environment * ev)
 {
 	OAF_ObjectDirectory retval;
@@ -183,20 +259,14 @@ OAF_ObjectDirectory_create (PortableServer_POA poa,
 	newservant->attr_domain = g_strdup (domain);
 	newservant->attr_hostID = oaf_hostname_get ();
 	newservant->by_iid = NULL;
-	newservant->attr_servers._buffer =
-		OAF_ServerInfo_load (source_directory,
-				     &newservant->attr_servers._length,
-				     &newservant->by_iid,
-				     g_get_user_name (),
-				     newservant->attr_hostID,
-				     newservant->attr_domain);
-	newservant->active_servers =
-		g_hash_table_new (g_str_hash, g_str_equal);
-	newservant->time_list_changed = time (NULL);
 
-#if defined(OAF_DEBUG) && 0
-	od_dump_list (newservant);
-#endif
+        newservant->registry_source_directories = g_strsplit (registry_path, ":", -1);
+        newservant->registry_directory_mtimes = g_hash_table_new (g_str_hash, g_str_equal);
+
+        update_registry (newservant);
+
+        newservant->active_servers =
+                g_hash_table_new (g_str_hash, g_str_equal);
 
 	return CORBA_Object_duplicate (retval, ev);
 }
@@ -208,6 +278,8 @@ impl_OAF_ObjectDirectory__get_servers (impl_POA_OAF_ObjectDirectory * servant,
 {
 	OAF_ServerInfoListCache *retval;
 
+        update_registry (servant);
+
 	retval = OAF_ServerInfoListCache__alloc ();
 
 	retval->_d = (only_if_newer < servant->time_list_changed);
@@ -295,6 +367,8 @@ impl_OAF_ObjectDirectory_activate (impl_POA_OAF_ObjectDirectory * servant,
 	OAF_ServerInfo *si;
 	ODActivationInfo ai;
 
+        update_registry (servant);
+
 	ai.ac = ac;
 	ai.flags = flags;
 	ai.ctx = ctx;
